@@ -2,7 +2,9 @@
 Client
 """
 import asyncio
+import logging
 import traceback
+from asyncio import IncompleteReadError
 from concurrent.futures.thread import ThreadPoolExecutor
 from typing import Dict, Optional, Callable
 
@@ -30,6 +32,7 @@ class KARPClient(object):
         self.writer: Optional[asyncio.StreamWriter] = None
 
         self.executor = ThreadPoolExecutor(max_workers=5)
+        self.logger = logging.getLogger("karp")
 
         self.routes: dict = {}
 
@@ -50,7 +53,7 @@ class KARPClient(object):
             successful = True
         except Exception as e:
             if req.response:
-                traceback.print_exc()
+                self.logger.debug(traceback.format_exc())
             response_data = e.__str__()
             successful = False
         if req.response:
@@ -70,13 +73,16 @@ class KARPClient(object):
                 if r.request_id in self._requests:
                     self._requests[r.request_id].complete(r)
         except InvalidResponseException:
-            traceback.print_exc()
+            self.logger.warning(traceback.print_exc())
 
     async def _read(self) -> None:
         while True:
             try:
-                responses: bytes = await self.reader.read(1000000)
-            except BrokenPipeError:
+                responses: bytes = await self.reader.readuntil(b"\n")
+                #  responses: bytes = await self.reader.read(1000000)
+            except (IncompleteReadError, BrokenPipeError):
+                break
+            except ConnectionResetError:
                 break
             if not responses:
                 break
@@ -84,6 +90,10 @@ class KARPClient(object):
             for response in responses.decode().split("\n"):
                 if response:
                     asyncio.ensure_future(self._handle(response))
+        for request in self._requests:
+            # Complete all the pending request to prevent being stuck
+            self._requests[request].complete(Response.create(request, "", False))
+        print("Disconnected from server.")
 
     def add_route(self, **kwargs) -> Callable:
         """
