@@ -5,6 +5,7 @@ KARP - Server
 import asyncio
 import logging
 import traceback
+from asyncio import IncompleteReadError, LimitOverrunError
 from concurrent.futures import ThreadPoolExecutor
 from typing import Callable, Optional, Dict
 
@@ -27,8 +28,12 @@ class Client:
     """
     Client
     """
+
     def __init__(
-        self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter, _id: str
+        self,
+        reader: asyncio.StreamReader,
+        writer: asyncio.StreamWriter,
+        _id: str,
     ):
         self.reader = reader
         self.writer = writer
@@ -116,7 +121,7 @@ class KARPServer:
             successful = True
         except (Exception, AssertionError) as e:
             if not req.response:
-                self.logger.warning(traceback.format_exc(e))
+                self.logger.debug(traceback.format_exc())
             response_data = e.__str__()
             successful = False
         if req.response:
@@ -124,9 +129,7 @@ class KARPServer:
             return res.to_bytes()
         return b""
 
-    def on_new_connection(
-        self, client: Client
-    ):
+    def on_new_connection(self, client: Client):
         """
         Called on new connection.
         :param client:
@@ -172,22 +175,40 @@ class KARPServer:
                     self.executor, self.on_new_connection, self.clients[_id]
                 )
             )
+        reader._limit = 2 ** 32
         self.logger.info(f"[+] New Connection from {_ip}")
         while not self.stopped:
-            try:
-                requests = await reader.read(1000000)
-            except (BrokenPipeError, ConnectionResetError):
+            complete = False
+            partial = b""
+            while not complete:
+                try:
+                    partial += await reader.readuntil(b"\n")
+                    complete = True
+                    # requests = await reader.read(1000000)
+                except (BrokenPipeError, ConnectionResetError):
+                    break
+                except IncompleteReadError:
+                    self.logger.warning(
+                        "A request was consumed, because the data stream ended without proper ending."
+                    )
+                    break
+                except LimitOverrunError as loe:
+                    self.logger.warning(
+                        f"A request was consumed, {loe.consumed} bytes was too long to handle."
+                    )
+                    new_part = await reader.read(loe.consumed)
+                    partial += new_part
+            if not partial:
                 break
-            if not requests:
-                break
-
             try:
-                for request in requests.decode().split("\n"):
+                for request in partial.decode().split("\n"):
                     if request:
                         self.logger.debug(f"= {request} =")
                         asyncio.ensure_future(
                             self._handle(request, writer, _id, _ip)
                         )
+                    else:
+                        self.logger.debug(f"No request = {request} =")
             except UnicodeDecodeError:
                 break
 
@@ -199,6 +220,8 @@ class KARPServer:
                     self.executor, self.on_connection_lost, self.clients[_id]
                 )
             )
+        if not self.clients[_id].writer.is_closing():
+            self.clients[_id].writer.close()
         del self.clients[_id]
         self.logger.info(f"[-] Disconnected from {_ip}")
 
